package meerkat.crypto.concrete.distributed_key_generation.communication;

import com.google.protobuf.Message;
import meerkat.crypto.utilitis.Channel;
import meerkat.protobuf.DKGMessages;

/**
 * Created by Tzlil on 2/14/2016.
 */
public abstract class MailHandler implements Channel.ReceiverCallback{

    public static final int BROADCAST = 0;
    private MessageHandler messageHandler;

    public MailHandler(MessageHandler messageHandler){
        this.messageHandler = messageHandler;
    }

    public abstract Message extractMessage(DKGMessages.Mail mail);

    public boolean isBroadcast(DKGMessages.Mail mail){
        return mail.getDestination() == BROADCAST;
    }

    public void receiveMail(DKGMessages.Mail mail){

        Message message = extractMessage(mail);
        if (message == null)
            return;

        switch (mail.getType()) {
            case SHARE:
                messageHandler.handleSecretMessage(mail.getSender(), isBroadcast(mail),message);
                break;
            case COMMITMENT:
                messageHandler.handleCommitmentMessage(mail.getSender(), isBroadcast(mail),message);
                break;
            case DONE:
                messageHandler.handleDoneMessage(mail.getSender(), isBroadcast(mail),message);
                break;
            case COMPLAINT:
                messageHandler.handleComplaintMessage(mail.getSender(), isBroadcast(mail),message);
                break;
            case ANSWER:
                messageHandler.handleAnswerMessage(mail.getSender(), isBroadcast(mail),message);
                break;
            case ABORT:
                messageHandler.handleAbortMessage(mail.getSender(), isBroadcast(mail),message);
                break;
            default:
                break;
        }


    }
}
